﻿


;-------------------------------------------------------------------------------
StartAnimation: ; start animation
;-------------------------------------------------------------------------------
    SetTimer, Animation, 1000

    ;-----------------------------
    Animation: ; timer controlled
    ;-----------------------------

    ; fill background
    Gdip_GraphicsClear(pGraphics)
    Gdip_FillRectangle(pGraphics, Brush.Back, 0, 0, Width, Height)

    ; draw dots with numerals
    s := Width / 6 ; size of dots behind numerals
    Loop, 12 {
        t := A_Index * Pi / 6
        Gdip_FillEllipse(pGraphics, Brush.Dots ; background
            , X := MidX - s/2 + Cos(t) * 0.4 * Width
            , Y := MidY - s/2 + Sin(t) * 0.4 * Height
            , s, s)
        Gdip_TextToGraphics(pGraphics, RomanNumerals(Mod(A_Index+2, 12) +1)
            , " x" X " y" Y " w" s " h" s " c" Format("{:x}", Colours.Num)
            . " s" Width / 800 * 48 " r" Mode " Center vCenter BoldItalic"
            , "Times New Roman")
    }

    ; draw 60 (some thick) marks
    Loop, 60 {
        t := A_Index * Pi / 30, d := Mod(A_Index, 5) ? 0.30 : 0.29
        Gdip_DrawLine(pGraphics, Mod(A_Index, 5) ? Pen.Mark : Pen.TMark
            , MidX + Cos(t) * d * Width
            , MidY + Sin(t) * d * Height
            , MidX + Cos(t) * 0.31 * Width
            , MidY + Sin(t) * 0.31 * Height)
    }

    ; draw all hands
    draw_Hand("Hour", Width/15, (A_Hour-3 + A_Min/60 + A_Sec/3600) * Pi/6, 0.25)
    draw_Hand("Min", Width/20, (A_Min-15 + A_Sec/60) * Pi/30, 0.35)
    draw_Hand("Sec", Width/40, (A_Sec-15) * Pi/30, 0.30)

    ; commit drawing
    UpdateLayeredWindow(hFace, hDC, 0, 0, A_ScreenWidth, A_ScreenHeight)

Return



;-------------------------------------------------------------------------------
draw_Hand(Which, s, t, len) { ; draw a hand extending from a plate
;-------------------------------------------------------------------------------
    ; s: size of the plate
    ; t: angle in radians
    ; len: length of the hand
    ;---------------------------------------------------------------------------
    global

    Gdip_FillEllipse(pGraphics, Brush[Which], MidX - s/2, MidY - s/2, s, s)
    Gdip_DrawLine(pGraphics, Pen[Which], MidX, MidY
        , MidX + Cos(t) * len * Width, MidY + Sin(t) * len * Height)
}



;-------------------------------------------------------------------------------
get_Colours(Style) { ; create Colours object from Style.ini
;-------------------------------------------------------------------------------
    global Colours := {}

    ItemList := ["Back", "Dots", "Num", "Mark", "TMark", "Hour", "Min", "Sec"]
    For each, Item in ItemList {
        IniRead, OutputVar, Styles\%Style%.ini, Colours, %Item%, %A_Space%
        Colours[Item] := OutputVar
    }
}



;-------------------------------------------------------------------------------
Zoom(Direction) { ; zoom in/out
;-------------------------------------------------------------------------------
    global

    SetTimer, Animation, Off

    Zoom += (Direction = "+") ? +0.05 : -0.05
    Zoom := Round(Clamp(Zoom, 0.15, 1.25), 2)
    ToolTip(Zoom, 8, 8)
    Gosub, delete_Gdip_Utensils
    Gosub, create_Gdip_Utensils

    MidX := (Width  := 800 * Zoom) / 2
    MidY := (Height := 800 * Zoom) / 2
    GuiControl, Move, %hParent%, w%Width% h%Height%
    Gui, Main: Show, AutoSize

    Gosub, StartAnimation
}



;-------------------------------------------------------------------------------
Clamp(Variable, Min, Max) { ; limit Variable to Min, Max respectively
;-------------------------------------------------------------------------------
	Return, (Variable < Min) ? Min
          : (Variable > Max) ? Max : Variable
}



;-------------------------------------------------------------------------------
ToolTip(Text, x, y) { ; wrapper
;-------------------------------------------------------------------------------
    ToolTip, %Text%, x, y
    SetTimer, ToolTipOff, -500
    Return

    ToolTipOff:
        ToolTip ; off
    Return
}



;-------------------------------------------------------------------------------
WM_LBUTTONDOWN() { ; move window
;-------------------------------------------------------------------------------
    static init := OnMessage(0x0201, "WM_LBUTTONDOWN")

    PostMessage, 0xA1, 2 ; WM_NCLBUTTONDOWN
}



;-------------------------------------------------------------------------------
WM_RBUTTONDOWN() { ; show context menu
;-------------------------------------------------------------------------------
    static init := OnMessage(0x0204, "WM_RBUTTONDOWN")

    Menu, ContextMenu, Show
}



;-------------------------------------------------------------------------------
RomanNumerals(n) { ; convert n to roman numerals
;-------------------------------------------------------------------------------
    If (n > 3999)
        Return, "Input too big."

    While n--
        Result .= "I"

    Result := StrReplace(Result, "IIIII", "V" )
    Result := StrReplace(Result, "IIII" , "IV")
    Result := StrReplace(Result, "VV"   , "X" )
    Result := StrReplace(Result, "VIV"  , "IX")
    Result := StrReplace(Result, "XXXXX", "L" )
    Result := StrReplace(Result, "XXXX" , "XL")
    Result := StrReplace(Result, "LL"   , "C" )
    Result := StrReplace(Result, "LXL"  , "XC")
    Result := StrReplace(Result, "CCCCC", "D" )
    Result := StrReplace(Result, "CCCC" , "CD")
    Result := StrReplace(Result, "DD"   , "M" )
    Result := StrReplace(Result, "DCD"  , "CM")

    Return, Result
}



;-------------------------------------------------------------------------------
CleanUp() { ; clean up
;-------------------------------------------------------------------------------
    global

    ; save position
    If Not IsMinimized {
        Gui, Main: +LastFound ; for WinGetPos
        WinGetPos, WinX, WinY, WinW, WinH
        Gosub, Write_INI
    }

    ; clean up Gdip
    Gosub, delete_Gdip_Utensils
    Gosub, teardown_Gdip

    ; done
    ExitApp
}
